import numpy as np
import scipy.stats as ss

def CalcMAPEst(samples, a = 0.05, n_points = 10000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """    
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]       
    return MAP_Est, est_pdf[map_idx]


f_phi = ss.norm.cdf    #   Cumulative standard normal distribution function


class PF:
    """
            Psychometric function with the parameters mu and sigma
    """
    def __init__(self, mu, sgm):
        self.mu = mu
        self.sgm = sgm

    def v(self, x):
        return f_phi((x - self.mu) / self.sgm)


def my_kernel(x, y, w_k):
    return np.exp(-((x - y) / w_k) ** 2)


def k_ave(v, X, W, w_k):
    n = len(X)
    num = 0.0
    den = 0.0
    for i in range(n):
        k_v = my_kernel(v, X[i], w_k)
        num += W[i] * k_v
        den += k_v
    return num / den
